"use strict";
var ddwcwm = jQuery.noConflict();
(() => {
	ddwcwm(document).on('ready', () => {
		if ( ddwcwm('#ddwcwm-enabled-payment-gateways').length ) {
			ddwcwm('#ddwcwm-enabled-payment-gateways').select2();
			ddwcwm('#ddwcwm-enabled-otp-operations').select2();
		}

		if ( ddwcwm('#ddwcwm-users').length ) {
			ddwcwm('#ddwcwm-users').select2({
				ajax: {
					method: 'post',
					url: ddwcwmAdminObj.ajax.ajaxUrl, // AJAX URL is predefined in WordPress admin
					dataType: 'json',
					delay: 250, // delay in ms while typing when to perform a AJAX search
					data: params => {
						return {
							query: params.term, // search query
							action: 'ddwcwm_get_users_list',
							ddwcwm_nonce: ddwcwmAdminObj.ajax.ajaxNonce,
						};
					},
					processResults: response => {
						let options = [];
						const users = response.users;
						if (users != undefined && users != null && users.length) {
							// users is the array of objects, and each of them contains value and the Label of the option
							users.forEach(user => {
								options.push({
									id: user.ID,
									text: `(#${user.ID}) ${user.user_login} <${user.user_email}>`
								});
							});
						}
						return {
							results: options
						};
					},
					cache: true
				},
				width : ddwcwm('#ddwcwm-transaction-from-date').length ? '20%' : '25em',
				minimumInputLength: 1,// the minimum of symbols to input before perform a search
				language: {
					inputTooShort: args => {
						const remain = args.minimum - args.input.length;
						return `${ddwcwmAdminObj.pleaseEnter} ${remain} ${ddwcwmAdminObj.moreCharacter}`;
					},
					noResults: () => ddwcwmAdminObj.noResult,
				}
			});
		}

		if ( ddwcwm('#ddwcwm-transaction-from-date').length ) {
			ddwcwm('#ddwcwm-transaction-from-date').datepicker({
				dateFormat: 'yy-mm-dd',
				onSelect: datetext => {
					ddwcwm('#ddwcwm-transaction-from-date').val(datetext);
				}
			});
		}

		if ( ddwcwm('#ddwcwm-transaction-to-date').length ) {
			ddwcwm('#ddwcwm-transaction-to-date').datepicker({
				dateFormat: 'yy-mm-dd',
				onSelect: datetext => {
					ddwcwm('#ddwcwm-transaction-to-date').val(datetext);
				}
			});
		}

		const ddwcwmRemoveElements = elms => {
			[...elms].forEach( el => {
				el.remove();
			});
		};

		ddwcwm(document).on('click', e => {
			if (e.target.classList.contains('ddwcwm-remove-general-rule-row') || e.target.classList.contains('ddwcwm-remove-product-row')) {
				ddwcwm(e.target).closest('tr').remove();
			}
		});

		if (ddwcwm('.ddwcwm-rule-wrap').length) {
			ddwcwm('#ddwcwm-add-product-rule-line').on('click', e => {
				e.preventDefault();
				let catRow = wp.template('ddwcwm-product-row');
				ddwcwm(e.target).closest('table').find('tbody').prepend(catRow());
				productSelect2();
			});

			ddwcwm('#ddwcwm-add-general-rule-line').on('click', e => {
				e.preventDefault();
				let ruleRow = wp.template('ddwcwm-general-rule-row');
				ddwcwm(e.target).closest('table').find('tbody').prepend(ruleRow());
			});

			ddwcwm('#ddwcwm-rule-form').on('submit', e => {
				let form = ddwcwm(e.target);
				ddwcwmRemoveElements(ddwcwm(".notice-error"));
				ddwcwmFormTraverse(form[0], e);
			});

			var productSelect2 = () => {
				if ( ddwcwm('.ddwcwm-product-select').length ) {

					ddwcwm('.ddwcwm-product-select').select2({
						width : '100%',
						ajax: {
							method: 'post',
							url: ddwcwmAdminObj.ajax.ajaxUrl, // AJAX URL is predefined in WordPress admin
							dataType: 'json',
							delay: 250, // delay in ms while typing when to perform a AJAX search
							data: params => {
								return {
									query: params.term, // search query
									action: 'ddwcwm_get_products_list',
									ddwcwm_nonce: ddwcwmAdminObj.ajax.ajaxNonce,
								};
							},
							processResults: products => {
								let options = [];
								if (products != undefined && products != null && products.length) {
									// users is the array of objects, and each of them contains value and the Label of the option
									products.forEach( product => {
										options.push({
											id: product.ID,
											text: product.title
										});
									} );
								}
								return {
									results: options
								};
							},
							cache: true
						},
						minimumInputLength: 1,// the minimum of symbols to input before perform a search
						language: {
							inputTooShort: args => {
								const remain = args.minimum - args.input.length;
								return `${ddwcwmAdminObj.pleaseEnter} ${remain} ${ddwcwmAdminObj.moreCharacter}`;
							},
							noResults: () => ddwcwmAdminObj.noResult,
						}
					});
				}
			}

			productSelect2();

			var ddwcwmFormTraverse = (form, e) => {
				var invalidFormFields = wp.template( 'ddwcwm_form_data_error' );
				let formContainsError = false;
				[...form.elements].forEach( input => {
					if( input.classList.contains( 'ddwcwm-product-select' ) ) {
						const select2ContainerElement = input.nextSibling;
						select2ContainerElement.querySelector( '.select2-selection' ).style.borderColor = '';
					}
					input.style.borderColor = '';
					let invalidData = false;
					var inputVal = input.value.trim();
					if( input.closest( '.ddwcwm-rule-wrap' ) && 'hidden' !== input.type ) {
						if ( -1 == inputVal || ! inputVal ) {
							e.preventDefault();
							if( input.classList.contains( 'ddwcwm-product-select' ) ) {
								const select2ContainerElement = input.nextSibling;
								select2ContainerElement.querySelector( '.select2-selection' ).style.borderColor = 'red';
							}
							input.style.borderColor = 'red';

							formContainsError = true;
						} else {

							var numFieldsClassLists = [ 'ddwcwm-product-quantity' ];

							var floatFields = [ 'ddwcwm-amount-from', 'ddwcwm-amount-to', 'ddwcwm-cashback-amount', 'ddwcwm-product-quantity', 'ddwcwm-product-cashback' ];

							if ( floatFields.indexOf(input.name) > -1 ) {
								if ( isNaN( inputVal ) ) {
									e.preventDefault();
									invalidData = true;
								}
							}

							if ( numFieldsClassLists.some( className => input.classList.contains( className ) ) ) {
								if (! /^\d+$/.test(inputVal)) {
									e.preventDefault();
									invalidData = true;
								}
							}

							if (/<\s?[^\>]*\/?\s?>/i.test(inputVal)) {
								e.preventDefault();
								invalidData = true;
							}

							if( invalidData ) {
								formContainsError = true;
								input.style.borderColor = 'red';
							}
						}
					}
				});
				if ( formContainsError ) {
					document.querySelector( '.ddwcwm-rule-wrap' ).insertAdjacentHTML('beforeBegin', invalidFormFields());
				}
			}

		}

		if ( ddwcwm( '.ddwcwm-verify-license' ).length ) {
			ddwcwm( '.ddwcwm-verify-license' ).on( 'click', e => {
				e.preventDefault();

				const licenseStatusElement = ddwcwm( '.ddwcwp-license-status' );
				const purchaseCodeElement = ddwcwm( '#ddwcwm-purchase-code' );
				const purchaseCode = purchaseCodeElement.val();
				const status = e.target.getAttribute( 'data-action' );

				const buttonLabel = e.target.innerHTML;

				e.target.innerHTML = e.target.getAttribute( 'data-fetching' );

				fetch( ddwcwmAdminObj.ajax.ajaxUrl, {
					method: 'POST',
					headers: new Headers( {
						'Content-Type': 'application/x-www-form-urlencoded',
						'Accept': 'application/json',
					} ),
					body: "action=ddwcwm_verify_license&ddwcwm_nonce=" + ddwcwmAdminObj.ajax.ajaxNonce + "&purchase_code=" + purchaseCode + "&status=" + status
				})
				.then( response => response.json() )
				.then( response => {
					e.target.innerHTML = buttonLabel;
					const licenseStatusParentElement = licenseStatusElement.parent( '.notice' );
					if ( response.success ) {
						ddwcwm( '.ddwcwm-verify-license' ).removeClass( 'ddwcwm-hide' );
						ddwcwm( e.target ).addClass( 'ddwcwm-hide' );
						licenseStatusParentElement.addClass( 'notice-success' );
						licenseStatusParentElement.removeClass( 'notice-error' );

						if ( status == 'activate' ) {
							purchaseCodeElement.addClass( 'ddwcwm-hide' );
						} else {
							purchaseCodeElement.removeClass( 'ddwcwm-hide' );
						}
					} else {
						licenseStatusParentElement.addClass( 'notice-error' );
						licenseStatusParentElement.removeClass( 'notice-success' );
					}
					licenseStatusElement.html( response.message );
					licenseStatusParentElement.removeClass( 'ddwcwm-hide' );
				} )
				.catch( err => {
					console.log( err );
				} );
			} );
		}

		if ( document.querySelectorAll( '.ddwcwm-view-withdraw-requests' ) ) {
			document.querySelectorAll( '.ddwcwm-view-withdraw-requests' ).forEach( element => {
				element.addEventListener( 'click', e => {
					e.preventDefault();
					const userId = e.target.getAttribute( 'data-user-id' );

					if ( document.querySelector( `#ddwcwm-wallet-withdraw-requests-${userId}` ) ) {
						document.querySelector( `#ddwcwm-wallet-withdraw-requests-${userId}` ).classList.add( 'ddwcwm-popup-open' );
					}
				} );
			} );
		}

		const ddwcwmClosePopup = element => {
			const popupElement = element.closest( '.ddwcwm-wallet-withdraw-requests-popup' );
			popupElement.classList.remove( 'ddwcwm-popup-open' );
			const messageElement = popupElement.querySelector( '.ddwcwm-message' );
			const messageParagraphElement = messageElement.querySelector( '.ddwcwm-message p' )
			messageParagraphElement.innerHTML = '';
			messageElement.classList.remove( 'ddwcwm-message-error' );
			messageElement.classList.remove( 'ddwcwm-message-success' );
			messageElement.classList.add( 'ddwcwm-hide' );
		}

		if ( document.querySelectorAll( '.ddwcwm-close-popup' ) ) {
			document.querySelectorAll( '.ddwcwm-close-popup' ).forEach( element => {
				element.addEventListener( 'click', e => {
					ddwcwmClosePopup( e.target );
				} );
			} );
		}

		if ( document.querySelectorAll( '.ddwcwm-wallet-withdraw-requests-popup' ) ) {
			document.querySelectorAll( '.ddwcwm-wallet-withdraw-requests-popup' ).forEach( element => {
				element.addEventListener( 'click', e => {
					if ( e.target.classList.contains( 'ddwcwm-wallet-withdraw-requests-popup' ) ) {
						ddwcwmClosePopup( e.target );
					}
				} );
			} );
		}

		if ( document.querySelectorAll( '.ddwcwm-withdraw-request-action' ) ) {
			document.querySelectorAll( '.ddwcwm-withdraw-request-action' ).forEach( element => {
				element.addEventListener( 'click', e => {
					e.preventDefault();
					e.target.setAttribute( 'disabled', 'disabled' );

					const userId = e.target.getAttribute( 'data-user-id' );
					const withdrawRequestId = e.target.getAttribute( 'data-withdraw-request-id' );
					const withdrawRequestPerform = e.target.getAttribute( 'data-withdraw-request-action' );
					const formData = new FormData();

					formData.append( 'action', 'ddwcwm_withdraw_request_action' );
					formData.append( 'ddwcwm_nonce', ddwcwmAdminObj.ajax.ajaxNonce );
					formData.append( 'user_id', userId );
					formData.append( 'withdraw_request_id', withdrawRequestId );
					formData.append( 'perform', withdrawRequestPerform );

					fetch( ddwcwmAdminObj.ajax.ajaxUrl, {
						method: "post",
						body: formData
					} )
					.then( response => response.ok ? response.json() : false ).then( response => {

						e.target.removeAttribute( 'disabled' );

						const messageElement = e.target.closest( '.ddwcwm-wallet-withdraw-requests-popup-content' ).querySelector( '.ddwcwm-message' );
						const messageParagraphElement = messageElement.querySelector( 'p' )
						messageParagraphElement.innerHTML = response.message;

						messageElement.classList.remove( 'ddwcwm-hide' );
						if ( response.success ) {
							messageElement.classList.add( 'ddwcwm-message-success' );
							messageElement.classList.remove( 'ddwcwm-message-error' );

							e.target.closest( 'tr' ).remove();
						} else {
							messageElement.classList.add( 'ddwcwm-message-error' );
							messageElement.classList.remove( 'ddwcwm-message-success' );
						}

					} );
				} );
			} );
		}

	});
})(ddwcwm);