<?php
/**
 * Admin functions class
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Admin;

use DDWCWalletManagement\Helper;
use DDWCWalletManagement\Helper\Transactions\DDWCWM_Transactions_Helper;
use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;
use DDWCWalletManagement\Templates\Admin;
use DDWCWalletManagement\Includes\Admin\Importer\DDWCWM_Users_Importer_Controller;
use DDWCWalletManagement\Includes\Admin\Exporter\DDWCWM_Users_CSV_Exporter;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Admin_Functions' ) ) {
	/**
	 * Admin functions
	 */
	class DDWCWM_Admin_Functions {
		/**
		 * Admin Menu
		 *
		 * @return void
		 */
		public function ddwcwm_add_menu_at_admin_end() {
			$menu_capability = apply_filters( 'ddwcwm_modify_menu_capability', 'manage_woocommerce' );

			add_menu_page( esc_html__( 'WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'WooCommerce Wallet', 'ddwc-wallet-management' ), $menu_capability, 'ddwc-wallet-management', [ $this, 'ddwcwm_get_users_list' ], 'dashicons-money', 56 );

			$hook_users_list = add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Users | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Users', 'ddwc-wallet-management' ), $menu_capability, 'ddwc-wallet-management', [ $this, 'ddwcwm_get_users_list' ] );

			add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Manual Credit/Debit | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Manual Credit/Debit', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-manual-credit-debit', [ $this, 'ddwcwm_get_manaul_credit_debit' ] );

			$hook_withdrawals_list = add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Withdrawal Requests | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Withdrawal Requests', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-withdraw-requests', [ $this, 'ddwcwm_get_withdraw_requests_list' ] );

			$hook_transactions_list = add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Transactions | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Transactions', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-transactions-list', [ $this, 'ddwcwm_get_transactions_list' ] );

			add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Cashback Rules | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Cashback Rules', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-cashback-rules', [ $this, 'ddwcwm_get_cashback_rules_list' ] );

			add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Configuration | WooCommerce Wallet', 'ddwc-wallet-management' ), esc_html__( 'Configuration', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-configuration', [ $this, 'ddwcwm_get_configuration' ] );

			add_submenu_page( 'ddwc-wallet-management', esc_html__( 'Extensions', 'ddwc-wallet-management' ), esc_html__( 'Extensions', 'ddwc-wallet-management' ), $menu_capability, 'ddwcwm-extensions', [ $this, 'ddwcwm_get_extensions' ] );

			do_action( 'ddwcwm_admin_menu_action' );

			add_action( "load-{$hook_users_list}", [ $this, 'ddwcwm_add_screen_options_in_users_list' ] );

			add_action( "load-{$hook_withdrawals_list}", [ $this, 'ddwcwm_add_screen_options_in_withdraw_requests_list' ] );

			add_action( "load-{$hook_transactions_list}", [ $this, 'ddwcwm_add_screen_options_in_transactions_list' ] );
		}

		/**
		 * Users List Screen Options.
		 *
		 * @return void
		 */
		public function ddwcwm_add_screen_options_in_users_list() {
			global $myListTable;

			$options = 'per_page';

			$args = [
				'label'   => esc_html__( 'Users Per Page', 'ddwc-wallet-management' ),
				'default' => 20,
				'option'  => 'users_per_page',
				'hidden'  => 'id'
			];

			add_screen_option( $options, $args );

			$myListTable = new Admin\Users\DDWCWM_Users_List_Template();
		}

		/**
		 * Withdrawal Requests List Screen Options.
		 *
		 * @return void
		 */
		public function ddwcwm_add_screen_options_in_withdraw_requests_list() {
			global $myListTable;

			$options = 'per_page';

			$args = [
				'label'   => esc_html__( 'Withdrawals Per Page', 'ddwc-wallet-management' ),
				'default' => 20,
				'option'  => 'withdrawals_per_page',
				'hidden'  => 'id'
			];

			add_screen_option( $options, $args );

			$myListTable = new Admin\Withdrawals\DDWCWM_Withdrawals_List_Template();
		}

		/**
		 * Transactions List Screen Options.
		 *
		 * @return void
		 */
		public function ddwcwm_add_screen_options_in_transactions_list() {
			global $myListTable;

			$options = 'per_page';

			$args = [
				'label'   => esc_html__( 'Transactions Per Page', 'ddwc-wallet-management' ),
				'default' => 20,
				'option'  => 'transactions_per_page',
				'hidden'  => 'id'
			];

			add_screen_option( $options, $args );

			$myListTable = new Admin\Transactions\DDWCWM_Transactions_List_Template();
		}

		/**
		 * Set Options.
		 *
		 * @param string $status Status.
		 * @param string $option Option.
		 * @param string $value Option Value.
		 * 
		 * @return string
		 */
		public function ddwcwm_set_options( $status, $option, $value ) {
			$options = [ 'users_per_page', 'withdrawals_per_page', 'transactions_per_page' ];
            return in_array( $option, $options, true ) ? $value : $status;
		}

		/**
		 * Set screen ids
		 *
		 * @param array $ids
		 * @return array
		 */
		public function ddwcwm_set_wc_screen_ids( $ids ) {
			$screen_id = sanitize_title( esc_html__( 'WooCommerce Wallet', 'ddwc-wallet-management' ) );
			$screen_ids = [
				'toplevel_page_' . $screen_id,
				'toplevel_page_ddwc-wallet-management',
				$screen_id . '_page_ddwcwm-manual-credit-debit',
				$screen_id . '_page_ddwcwm-cashback-rules',
				$screen_id . '_page_ddwcwm-configuration',
			];

			array_push( $ids, ...$screen_ids );
			return $ids;
		}

		/**
		 * Register Option settings
		 *
		 * @return void
		 */
		public function ddwcwm_register_settings() {
			if ( ! empty( $_GET[ 'action' ] ) && $_GET[ 'action' ] == 'ddwcwm-users-export' && isset( $_GET[ 'handle' ], $_GET[ 'nonce' ] ) && wp_verify_nonce( wp_unslash( $_GET[ 'nonce' ] ), 'users-csv' ) && 'download_users_csv' === wp_unslash( $_GET[ 'handle' ] ) ) { // WPCS: input var ok, sanitization ok.
				$exporter = new DDWCWM_Users_CSV_Exporter();

				if ( ! empty( $_GET[ 'filename' ] ) ) { // WPCS: input var ok.
					$exporter->set_filename( wp_unslash( $_GET[ 'filename' ] ) ); // WPCS: input var ok, sanitization ok.
				}

				$exporter->export();
			}

			$sanitize = [
				'sanitize_callback' => [ $this, 'ddwcwm_sanitize_option_value' ],
			];

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_enabled' );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_min_topup_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_max_topup_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_debit_limit_type', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_min_debit_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_max_debit_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_min_transfer_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_max_transfer_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_registration_credit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_topup_order_status', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_partial_payments_enabled', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_enabled_payment_gateways' );

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_enabled_otp_operations' );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_otp_expiry', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_otp_length', $sanitize );

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_withdrawals_enabled' );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_withdraw_charges_type', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_withdraw_charges_amount', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_min_withdrawal_limit', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_max_withdrawal_limit', $sanitize );

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_referrals_enabled' );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_referral_earning_amount', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_referral_email_content', $sanitize );

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_my_account_endpoint', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_my_account_endpoint_title', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_enable_widgets_my_account_endpoint', $sanitize );

			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_wallet_balance_only_shortcode', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_wallet_balance_shortcode', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_wallet_operations_shortcode', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_wallet_balance_operations_shortcode', $sanitize );
			register_setting( 'ddwcwm-configuration-fields', '_ddwcwm_wallet_transactions_shortcode', $sanitize );
		}

		/**
		 * Admin scripts
		 *
		 * @return void
		 */
		public function ddwcwm_admin_scripts() {
			$pages = [ 'ddwc-wallet-management', 'ddwcwm-manual-credit-debit', 'ddwcwm-withdraw-requests', 'ddwcwm-transactions-list', 'ddwcwm-cashback-rules', 'ddwcwm-configuration' ];

			$pages = apply_filters( 'ddwcwm_modify_pages_to_enqueue_script_at_admin_end', $pages );

			if ( ( ! empty( $_REQUEST[ 'page' ] ) && in_array( $_REQUEST[ 'page' ], $pages ) ) || apply_filters( 'ddwcwm_enqueue_admin_scripts_at_custom_page', false ) ) {
				if ( ! empty( $_REQUEST[ 'action' ] ) && $_REQUEST[ 'action' ] == 'ddwcwm-users-export' ) {
					wp_register_script( 'ddwcwm-users-export', DDWCWM_PLUGIN_URL . 'assets/js/users-export.js', [ 'jquery', 'wc-enhanced-select' ], DDWCWM_SCRIPT_VERSION );
					wp_localize_script(
						'ddwcwm-users-export',
						'ddwcwm_users_export_params',
						[
							'export_nonce' => wp_create_nonce( 'ddwcwm-users-export' ),
						]
					);
				}

				if ( ! empty( $_REQUEST[ 'action' ] ) && $_REQUEST[ 'action' ] == 'ddwcwm-users-import' ) {
					wp_register_script( 'ddwcwm-users-import', DDWCWM_PLUGIN_URL . 'assets/js/users-import.js', [ 'jquery' ], DDWCWM_SCRIPT_VERSION );
				}

				wp_enqueue_style( 'ddwcwm-admin-style', DDWCWM_PLUGIN_URL . 'assets/css/admin.css', [], DDWCWM_SCRIPT_VERSION );

				wp_enqueue_style( 'select2', plugins_url() . '/woocommerce/assets/css/select2.css' );

				wp_enqueue_script( 'ddwcwm-admin-script', DDWCWM_PLUGIN_URL . 'assets/js/admin.js', [ 'select2', 'wp-util' ], DDWCWM_SCRIPT_VERSION );

				wp_enqueue_script( 'jquery-ui-datepicker' );

				wp_enqueue_style( 'ddwcwm-jquery-ui-style', DDWCWM_PLUGIN_URL . 'assets/css/jquery-ui.css', [], DDWCWM_SCRIPT_VERSION );

				wp_localize_script( 'ddwcwm-admin-script', 'ddwcwmAdminObj', [
					'ajax' => [
						'ajaxUrl'   => admin_url( 'admin-ajax.php' ),
						'ajaxNonce' => wp_create_nonce( 'ddwcwm-nonce' ),
					],
					'pleaseEnter'      => esc_html__( 'Please enter', 'ddwc-wallet-management' ),
					'moreCharacter'    => esc_html__( 'or more character', 'ddwc-wallet-management' ),
					'noResult'         => esc_html__( 'No result Found', 'ddwc-wallet-management' ),
				] );
			}
		}

		/**
		 * Sanitize and Unslash Option Input
		 *
		 * @param string $input Input String.
		 * @return $input
		 */
		public function ddwcwm_sanitize_option_value( $input ) {
			$input = sanitize_text_field( wp_unslash( $input ) );
			return $input;
		}

		/**
		 * Users List Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_users_list() {
			if ( ! empty( $_GET[ 'action' ] ) ) {
				if ( $_GET[ 'action' ] === 'ddwcwm-users-import' ) {
					if ( defined( 'WP_LOAD_IMPORTERS' ) ) {
						wp_safe_redirect( admin_url( "admin.php?&page=${$_GET['page']}&action=ddwcwm-users-import" ) );
						exit;
					}

					$importer = new DDWCWM_Users_Importer_Controller();
					$importer->dispatch();
				} else if ( $_GET[ 'action' ] === 'ddwcwm-users-export' ) {
					wp_enqueue_script( 'ddwcwm-users-export' );

					$exporter = new DDWCWM_Users_CSV_Exporter();
					?>
					<div class="wrap woocommerce">
						<h1 class="wp-heading-inline"><?php esc_html_e( 'Export Users', 'ddwc-wallet-management' ); ?></h1>
						<a href="<?php echo esc_url( admin_url( 'admin.php?page=' . $_GET[ 'page' ] ) ); ?>" class="page-title-action"><?php esc_html_e( 'Back', 'ddwc-wallet-management' ); ?></a>
						<hr class="wp-header-end" />

						<div class="woocommerce-exporter-wrapper">
							<form class="woocommerce-exporter">
								<header>
									<span class="spinner is-active"></span>
									<h2><?php esc_html_e( 'Export users to a CSV file', 'ddwc-wallet-management' ); ?></h2>
									<p><?php esc_html_e( 'This tool allows you to generate and download a CSV file containing a list of all users.', 'ddwc-wallet-management' ); ?></p>
								</header>
								<section>
									<table class="form-table woocommerce-exporter-options">
										<tbody>
											<tr>
												<th scope="row">
													<label for="woocommerce-exporter-columns"><?php esc_html_e( 'Which columns should be exported?', 'ddwc-wallet-management' ); ?></label>
												</th>
												<td>
													<select id="woocommerce-exporter-columns" class="woocommerce-exporter-columns wc-enhanced-select" style="width:100%;" multiple data-placeholder="<?php esc_attr_e( 'Export all columns', 'ddwc-wallet-management' ); ?>">
														<?php
														foreach ( $exporter->get_default_column_names() as $column_id => $column_name ) {
															echo '<option value="' . esc_attr( $column_id ) . '">' . esc_html( $column_name ) . '</option>';
														}
														?>
													</select>
												</td>
											</tr>
										</tbody>
									</table>
									<progress class="woocommerce-exporter-progress" max="100" value="0"></progress>
								</section>
								<div class="wc-actions">
									<button type="submit" class="woocommerce-exporter-button button button-primary" value="<?php esc_attr_e( 'Generate CSV', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'Generate CSV', 'ddwc-wallet-management' ); ?></button>
								</div>
							</form>
						</div>
					</div>
					<?php
				}
			} else {
				$obj = new Admin\Users\DDWCWM_Users_List_Template();
				?>
				<div class="wrap">
					<h1 class="wp-heading-inline"><?php esc_html_e( 'Users', 'ddwc-wallet-management' ); ?></h1>
					<a href="<?php echo esc_url( admin_url( 'admin.php?page=ddwcwm-manual-credit-debit' ) ); ?>" class="page-title-action"><?php esc_html_e( 'Manual Credit/Debit', 'ddwc-wallet-management' ); ?></a>
					<a href="<?php echo esc_url( admin_url( "admin.php?page={$_GET['page']}&action=ddwcwm-users-import" ) ); ?>" class="page-title-action"><?php esc_html_e( 'Import', 'ddwc-wallet-management' ); ?></a>
					<a href="<?php echo esc_url( admin_url( "admin.php?page={$_GET['page']}&action=ddwcwm-users-export" ) ); ?>" class="page-title-action"><?php esc_html_e( 'Export', 'ddwc-wallet-management' ); ?></a>
					<hr class="wp-header-end" />
					<form method="get">
						<input type="hidden" name="page" value="<?php echo isset( $_REQUEST[ 'page' ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_REQUEST[ 'page' ] ) ) ) : ''; // WPCS: CSRF ok. // WPCS: input var ok. ?>" />
						<?php
						wp_nonce_field( 'ddwcwm_users_list_nonce_action', 'ddwcwm_users_list_nonce' );
						$obj->prepare_items();
						$obj->search_box( esc_html__( 'Search', 'ddwc-wallet-management' ), 'search-id' );
						$obj->display();
						?>
					</form>
				</div>
				<?php
			}
		}

		/**
		 * Get Manual Credit/Debit function
		 *
		 * @return void
		 */
		public function ddwcwm_get_manaul_credit_debit() {
			new Admin\Manual_Credit_Debit\DDWCWM_Manual_Credit_Debit_Template();
		}

		/**
		 * Withdraw Requests Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_withdraw_requests_list() {
			$obj = new Admin\Withdrawals\DDWCWM_Withdrawals_List_Template();
			?>
			<div class="wrap">
				<h1 class="wp-heading-inline"><?php esc_html_e( 'Withdrawal Requests', 'ddwc-wallet-management' ); ?></h1>
				<hr class="wp-header-end" />
				<form method="get">
					<input type="hidden" name="page" value="<?php echo isset( $_REQUEST['page'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_REQUEST['page'] ) ) ) : ''; // WPCS: CSRF ok. // WPCS: input var ok. ?>" />
					<?php
					wp_nonce_field( 'ddwcwm_users_list_nonce_action', 'ddwcwm_users_list_nonce' );
					$obj->prepare_items();
					$obj->search_box( esc_html__( 'Search', 'ddwc-wallet-management' ), 'search-id' );
					$obj->display();
					?>
				</form>
			</div>
			<?php
		}

		/**
		 * Transactions List Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_transactions_list() {
			?>
			<div class="wrap">
				<?php
				if ( ! empty( $_GET[ 'action' ] ) && 'view' === sanitize_key( $_GET[ 'action' ] ) && ! empty( $_GET[ 'transaction_id' ] ) ) { // wpcs: input var okay. wpcs: csrf okay.
					$obj = new Helper\Transactions\DDWCWM_Transactions_Helper();
					$obj->ddwcwm_get_transaction_template( $_GET[ 'transaction_id' ] );
				} else {
					$obj = new Admin\Transactions\DDWCWM_Transactions_List_Template();
					?>
					<h1 class="wp-heading-inline"><?php esc_html_e( 'Transactions', 'ddwc-wallet-management' ); ?></h1>
					<hr class="wp-header-end" />
					<form method="get">
						<input type="hidden" name="page" value="<?php echo isset( $_REQUEST[ 'page' ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_REQUEST[ 'page' ] ) ) ) : ''; // WPCS: CSRF ok. // WPCS: input var ok. ?>" />
						<?php
						wp_nonce_field( 'ddwcwm_transactions_list_nonce_action', 'ddwcwm_transactions_list_nonce' );
						$obj->prepare_items();
						$obj->search_box( esc_html__( 'Search', 'ddwc-wallet-management' ), 'search-id' );
						$obj->display();
						?>
					</form>
					<?php
				}
				?>
			</div>
			<?php
		}

		/**
		 * Transactions List Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_cashback_rules_list() {
			new Admin\Rules\DDWCWM_Cashback_Rules_Template();
		}

		/**
		 * Configuration Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_configuration() {
			new Admin\Configuration\DDWCWM_Configuration_Template();
		}

		/**
		 * Extensions Submenu
		 *
		 * @return void
		 */
		public function ddwcwm_get_extensions() {
			wp_redirect( '//devdiggers.com/store/' );
			exit();
		}

		/**
		 * Handle wallet on order cancel function
		 *
		 * @param int $order_id
		 * @return void
		 */
		public function ddwcwm_handle_wallet_on_order_cancelled( $order_id ) {
			$order_cancellation_handled = get_post_meta( $order_id, '_ddwcwm_order_cancellation_handled', true );

			if ( empty( $order_cancellation_handled ) ) {
				$order = wc_get_order( $order_id );

				$wallet_amount_used = get_post_meta( $order_id, '_ddwcwm_wallet_amount_used', true );

				$user_helper = new DDWCWM_Users_Helper();
				$transaction_helper = new DDWCWM_Transactions_Helper();

				$user_id = get_post_meta( $order_id, '_customer_user', true );

				$wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );

				if ( ! empty( $wallet_amount_used ) ) {
					do_action( 'ddwcwm_before_credit_wallet_amount_on_order_cancelled', $wallet_amount_used, $user_id, $order_id );

					$wallet_balance += $wallet_amount_used;

					$data = [
						'type'      => 'credit',
						'amount'    => $wallet_amount_used,
						'user_id'   => $user_id,
						'order_id'  => $order_id,
						'note'      => esc_html__( 'Credited wallet amount on order cancel.', 'ddwc-wallet-management' ),
						'date'      => current_time( 'Y-m-d H:i:s' ),
						'reference' => 'order_cancelled_credit',
					];

					$transaction_helper->ddwcwm_save_transaction( $data );

					$email_message = [];

					$email_message[] = sprintf( esc_html__( 'You have been credited with %s wallet amount which you have used in order %s due to its cancellation.', 'ddwc-wallet-management' ), wc_price( $wallet_amount_used ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

					$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( $wallet_balance ) );

					$email_data = [
						'email'   => get_userdata( $user_id )->user_email,
						'message' => $email_message
					];

					do_action( 'ddwcwm_mail', $email_data );
				}

				$cashback_amount       = get_post_meta( $order_id, '_ddwcwm_cashback_amount', true );
				$awarded_cashback_data = get_post_meta( $order_id, '_ddwcwm_awarded_cashback_data', true );

				if ( ! empty( $awarded_cashback_data ) && ! empty( $cashback_amount ) ) {
					$cashback_amount = 0;

					if ( ! empty( $awarded_cashback_data[ 'cart' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'cart' ];

						$wallet_balance -= $awarded_cashback_data[ 'cart' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'cart' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for cart due to order cancel.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'cart_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'product' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'product' ];

						$wallet_balance -= $awarded_cashback_data[ 'product' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'product' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for products due to order cancel.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'product_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'topup' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'topup' ];

						$wallet_balance -= $awarded_cashback_data[ 'topup' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'topup' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for topup due to order cancel.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'topup_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $cashback_amount ) ) {
						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your wallet is debited with %s cashback for order %s due to its cancellation.', 'ddwc-wallet-management' ), wc_price( $cashback_amount ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

						$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( $wallet_balance ) );

						$email_data = [
							'email'   => get_userdata( $user_id )->user_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );
					}
				}

				$user_helper->ddwcwm_set_user_wallet_balance( $wallet_balance, $user_id );

				update_post_meta( $order_id, '_ddwcwm_order_cancellation_handled', true );
			}
		}

		/**
		 * Handle wallet on order fully refunded function
		 *
		 * @param int $order_id
		 * @return void
		 */
		public function ddwcwm_handle_wallet_on_order_fully_refunded( $order_id ) {
			$order = wc_get_order( $order_id );

			$order_refunded_handled = get_post_meta( $order_id, '_ddwcwm_order_fully_refund_handled', true );

			if ( empty( $order_refunded_handled ) ) {
				$order = wc_get_order( $order_id );

				$wallet_amount_used = get_post_meta( $order_id, '_ddwcwm_wallet_amount_used', true );

				$user_helper        = new DDWCWM_Users_Helper();
				$transaction_helper = new DDWCWM_Transactions_Helper();

				$user_id = get_post_meta( $order_id, '_customer_user', true );

				$wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );

				if ( ! empty( $wallet_amount_used ) ) {
					do_action( 'ddwcwm_before_credit_wallet_amount_on_order_refunded', $wallet_amount_used, $user_id, $order_id );

					$new_wallet_balance = $wallet_balance + $wallet_amount_used;

					$data = [
						'type'      => 'credit',
						'amount'    => $wallet_amount_used,
						'user_id'   => $user_id,
						'order_id'  => $order_id,
						'note'      => esc_html__( 'Credited wallet amount on order fully refund.', 'ddwc-wallet-management' ),
						'date'      => current_time( 'Y-m-d H:i:s' ),
						'reference' => 'order_refund',
					];

					$transaction_helper->ddwcwm_save_transaction( $data );

					$email_message = [];

					$email_message[] = sprintf( esc_html__( 'You have been credited with %s wallet amount which you have used in order %s due to its fully refund.', 'ddwc-wallet-management' ), wc_price( $wallet_amount_used ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

					$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( $new_wallet_balance ) );

					$email_data = [
						'email' => get_userdata( $user_id )->user_email,
						'message' => $email_message
					];

					do_action( 'ddwcwm_mail', $email_data );
				}

				$cashback_amount       = get_post_meta( $order_id, '_ddwcwm_cashback_amount', true );
				$awarded_cashback_data = get_post_meta( $order_id, '_ddwcwm_awarded_cashback_data', true );

				if ( ! empty( $awarded_cashback_data ) && ! empty( $cashback_amount ) ) {
					$cashback_amount = 0;

					if ( ! empty( $awarded_cashback_data[ 'cart' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'cart' ];

						$wallet_balance -= $awarded_cashback_data[ 'cart' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'cart' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for cart due to order fully refund.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'cart_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'product' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'product' ];

						$wallet_balance -= $awarded_cashback_data[ 'product' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'product' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for products due to order fully refund.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'product_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'topup' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'topup' ];

						$wallet_balance -= $awarded_cashback_data[ 'topup' ];

						$data = [
							'type'      => 'debit',
							'amount'    => $awarded_cashback_data[ 'topup' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Debit wallet cashback for topup due to order fully refund.', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'topup_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $cashback_amount ) ) {
						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your wallet is debited with %s cashback for order %s due to its fully refund.', 'ddwc-wallet-management' ), wc_price( $cashback_amount ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

						$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( $wallet_balance ) );

						$email_data = [
							'email'   => get_userdata( $user_id )->user_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );
					}
				}

				if ( ! empty( $new_wallet_balance ) ) {
					$user_helper->ddwcwm_set_user_wallet_balance( $new_wallet_balance, $user_id );
				}

				update_post_meta( $order_id, '_ddwcwm_order_fully_refund_handled', true );
			}
		}
	}
}
