<?php
/**
 * Front ajax functions class
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Front;

use DDWCWalletManagement\Helper\Transactions\DDWCWM_Transactions_Helper;
use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Front_Ajax_Functions' ) ) {
	/**
	 * Front Ajax fuctions class
	 */
	class DDWCWM_Front_Ajax_Functions {

		/**
		 * Add Wallet amount in session function
		 *
		 * @return void
		 */
		public function ddwcwm_add_wallet_amount_in_session() {
			$response = [];
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				global $ddwcwm_wallet;
				$user_helper         = new DDWCWM_Users_Helper();
				$user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance();
				$cart_total          = apply_filters( 'ddwcwm_modify_amount_to_base_currency', floatval( WC()->cart->total ) );

				if ( 'percentage' === $ddwcwm_wallet[ 'debit_limit_type' ] ) {
					$ddwcwm_wallet[ 'min_debit_limit' ] = floatval( $ddwcwm_wallet[ 'min_debit_limit' ] ) * floatval( $cart_total ) / 100;
					$ddwcwm_wallet[ 'max_debit_limit' ] = floatval( $ddwcwm_wallet[ 'max_debit_limit' ] ) * floatval( $cart_total ) / 100;
				}

				if ( ! empty( $ddwcwm_wallet[ 'max_debit_limit' ] ) ) {
					if ( $ddwcwm_wallet[ 'max_debit_limit' ] <= $cart_total && $ddwcwm_wallet[ 'max_debit_limit' ] <= $user_wallet_balance ) {
						$wallet_amount_used = $ddwcwm_wallet[ 'max_debit_limit' ];
					} else if ( $ddwcwm_wallet[ 'max_debit_limit' ] <= $cart_total && $ddwcwm_wallet[ 'max_debit_limit' ] > $user_wallet_balance ) {
						$wallet_amount_used = $user_wallet_balance;
					} else {
						$wallet_amount_used = $user_wallet_balance;
					}
				} else {
					$wallet_amount_used = $user_wallet_balance;
				}

				$wallet_amount_used = apply_filters( 'ddwcwm_modify_fee_wallet_amount_in_session', $wallet_amount_used );

				if ( ! empty( $wallet_amount_used ) && ! empty( WC()->session ) && is_object( WC()->session ) ) {
					WC()->session->set( 'ddwcwm_wallet_amount', $wallet_amount_used );

					$response = [
						'success' => true,
						'message' => esc_html__( 'Partial Wallet is applied successfully.', 'ddwc-wallet-management' ),
					];
				}
			} else {
				$response = [
					'success'   => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}
			wp_send_json( $response );
		}

		/**
		 * Remove Wallet from Session
		 *
		 * @return void
		 */
		public function ddwcwm_remove_wallet_amount_from_session() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				WC()->session->__unset( 'ddwcwm_wallet_amount' );

				$response = [
					'success' => true,
					'message' => esc_html__( 'Partial Wallet has been removed successfully.', 'ddwc-wallet-management' ),
				];
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
		 * Send money to user
		 *
		 * @return void
		 */
		public function ddwcwm_send_money_to_user() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				$email = ! empty( $_POST[ 'ddwcwm_email' ] ) && is_email( $_POST[ 'ddwcwm_email' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_email' ] ) : '';
                $amount = ! empty( $_POST[ 'ddwcwm_amount' ] ) ? floatval( sanitize_text_field( $_POST[ 'ddwcwm_amount' ] ) ) : '';
				$note = ! empty( $_POST[ 'ddwcwm_note' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_note' ] ) : '';

				$amount = apply_filters( 'ddwcwm_modify_amount_to_base_currency', $amount );

				if ( empty( $email ) || empty( $amount ) ) {
					$response = [
						'success' => false,
						'message' => esc_html__( 'Email or amount fields are either empty or invalid.', 'ddwc-wallet-management' ),
					];
				} else {
					if ( ! email_exists( $email ) ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Email you entered is not exists.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					$user = wp_get_current_user();

					if ( $user->user_email === $email ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Cannot transfer money to yourself.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					global $ddwcwm_wallet;

					$user_helper = new DDWCWM_Users_Helper();
					$current_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance();

					if ( empty( $error) && $amount > $current_user_wallet_balance ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'You do not have enough wallet balance to send.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'min_transfer_limit' ] ) && $amount < $ddwcwm_wallet[ 'min_transfer_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be more than the min transfer amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'max_transfer_limit' ] ) && $amount > $ddwcwm_wallet[ 'max_transfer_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be less than the max transfer amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error ) ) {
						$current_user_wallet_balance -= $amount;

						$user_id = get_current_user_id();

						$user_helper->ddwcwm_set_user_wallet_balance( $current_user_wallet_balance, $user_id );

						do_action( 'ddwcwm_after_send_money_from_wallet', $user_id, $amount );

						$receiver_id = get_user_by( 'email', $email )->ID;

						$receiver_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $receiver_id );

						$receiver_user_wallet_balance += $amount;

						$user_helper->ddwcwm_set_user_wallet_balance( $receiver_user_wallet_balance, $receiver_id );

						$data = [
							'type'      => 'transfer',
							'amount'    => $amount,
							'sender_id' => $user_id,
							'user_id'   => $receiver_id,
							'note'      => $note,
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'wallet_transfer',
						];

						$transaction_helper = new DDWCWM_Transactions_Helper();

						$transaction_helper->ddwcwm_save_transaction( $data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( '%s amount is debited from your wallet balance for the wallet transfer.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Note For Receiver: %s', 'ddwc-wallet-management' ), $note );
						}

						$email_message[] = esc_html__( 'Available Balance:', 'ddwc-wallet-management' ) . ' ' . wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $current_user_wallet_balance ) );

						$sender_email = get_userdata( $user_id )->user_email;

						$email_data = [
							'email'   => $sender_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( '%s has transferred amount %s in your wallet balance.', 'ddwc-wallet-management' ), $sender_email, wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Note From Sender: %s', 'ddwc-wallet-management' ), $note );
						}

						$email_message[] = esc_html__( 'Available Balance:', 'ddwc-wallet-management' ) . ' ' . wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $receiver_user_wallet_balance ) );

						$email_data = [
							'email'   => $email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$response = [
							'success' => true,
							'message' => sprintf( esc_html__( 'Wallet amount %s transferred successfully.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) ),
						];
					}
				}
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
		 * Request money from user
		 *
		 * @return void
		 */
		public function ddwcwm_request_money_from_user() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				$email = ! empty( $_POST[ 'ddwcwm_email' ] ) && is_email( $_POST[ 'ddwcwm_email' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_email' ] ) : '';
                $amount = ! empty( $_POST[ 'ddwcwm_amount' ] ) ? floatval( sanitize_text_field( $_POST[ 'ddwcwm_amount' ] ) ) : '';
				$note = ! empty( $_POST[ 'ddwcwm_note' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_note' ] ) : '';

				$amount = apply_filters( 'ddwcwm_modify_amount_to_base_currency', $amount );

				if ( empty( $email ) || empty( $amount ) ) {
					$response = [
						'success' => false,
						'message' => esc_html__( 'Email or amount fields are either empty or invalid.', 'ddwc-wallet-management' ),
					];
				} else {
					if ( ! email_exists( $email ) ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Email you entered is not exists.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}
					global $ddwcwm_wallet;

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'min_transfer_limit' ] ) && $amount < $ddwcwm_wallet[ 'min_transfer_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be more than the min transfer amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'max_transfer_limit' ] ) && $amount > $ddwcwm_wallet[ 'max_transfer_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be less than the max transfer amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error ) ) {
						$user_id = get_current_user_id();

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'You have successfully requested amount %s from %s. You\'ll get notified if he/she will send the amount to you.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ), '<a href="mailto:' . esc_attr( $email ) . '">' . esc_html( $email ) . '</a>' );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Request Note: %s', 'ddwc-wallet-management' ), $note );
						}

						$sender_email = get_userdata( $user_id )->user_email;

						$email_data = [
							'email'   => $sender_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'You\'ve got a wallet money request of amount %s from %s. You can send him/her money by using send money option from My Wallet page.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ), '<a href="mailto:' . esc_attr( $sender_email ) . '">' . esc_html( $sender_email ) . '</a>' );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Request Note: %s', 'ddwc-wallet-management' ), $note );
						}

						$email_data = [
							'email'   => $email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$response = [
							'success' => true,
							'message' => sprintf( esc_html__( 'Wallet amount %s requested successfully.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) ),
						];
					}
				}
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
		 * Withdraw money request function
		 *
		 * @return void
		 */
		public function ddwcwm_withdraw_money_from_user() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
                $amount = ! empty( $_POST[ 'ddwcwm_amount' ] ) ? floatval( sanitize_text_field( $_POST[ 'ddwcwm_amount' ] ) ) : '';
				$note = ! empty( $_POST[ 'ddwcwm_note' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_note' ] ) : '';

				$amount = apply_filters( 'ddwcwm_modify_amount_to_base_currency', $amount );

				if ( empty( $amount ) || empty( $note ) ) {
					$response = [
						'success' => false,
						'message' => esc_html__( 'Amount or note fields are either empty or invalid.', 'ddwc-wallet-management' ),
					];
				} else {
					global $ddwcwm_wallet;

					$user_helper = new DDWCWM_Users_Helper();
					$current_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance();

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'min_withdrawal_limit' ] ) && $amount < $ddwcwm_wallet[ 'min_withdrawal_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be more than the min withdrawal amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error) && ! empty( $ddwcwm_wallet[ 'max_withdrawal_limit' ] ) && $amount > $ddwcwm_wallet[ 'max_withdrawal_limit' ] ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'Amount should be less than the max withdrawal amount.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error) && $amount > $current_user_wallet_balance ) {
						$response = [
							'success' => false,
							'message' => esc_html__( 'You do not have enough wallet balance to withdraw.', 'ddwc-wallet-management' ),
						];
						$error = 1;
					}

					if ( empty( $error ) ) {
						$current_user_wallet_balance -= $amount;
						$user_id = get_current_user_id();

						$user_helper->ddwcwm_set_user_wallet_balance( $current_user_wallet_balance, $user_id );

						do_action( 'ddwcwm_after_withdraw_money_from_wallet', $user_id, $amount );

						$charges = 0;

						if ( ! empty( $ddwcwm_wallet[ 'withdraw_charges_amount' ] ) ) {
							if ( $ddwcwm_wallet[ 'withdraw_charges_type' ] == 'fixed' ) {
								$charges = $ddwcwm_wallet[ 'withdraw_charges_amount' ];
							} else if ( $ddwcwm_wallet[ 'withdraw_charges_type' ] == 'percentage' ) {
								$charges = $ddwcwm_wallet[ 'withdraw_charges_amount' ] * $amount / 100;
							}
						}

						$withdraw_amount_after_charges = $amount - $charges;

						$data = [
							'type'      => 'withdraw',
							'amount'    => $withdraw_amount_after_charges,
							'user_id'   => $user_id,
							'note'      => $note,
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'withdraw_requested',
						];

						$transaction_helper = new DDWCWM_Transactions_Helper();

						$transaction_helper->ddwcwm_save_transaction( $data );

						if ( ! empty( $charges ) ) {
							$data = [
								'type'      => 'withdraw',
								'amount'    => $charges,
								'user_id'   => $user_id,
								'note'      => $note,
								'date'      => current_time( 'Y-m-d H:i:s' ),
								'reference' => 'withdraw_charges',
							];
							$transaction_helper->ddwcwm_save_transaction( $data );
						}

						$withdraw_requests = $user_helper->ddwcwm_get_user_withdraw_requests();

						$withdraw_requests[] = [
							'amount_before_charges' => $amount,
							'charges'               => $charges,
							'amount_after_charges'  => $withdraw_amount_after_charges,
							'date'                  => current_time( 'Y-m-d H:i:s' ),
							'note'                  => $note,
						];

						$user_helper->ddwcwm_set_user_withdraw_requests( $withdraw_requests );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'You have successfully requested amount %s for the withdrawal. You\'ll get notified via email when admin pay the same.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) );

						if ( ! empty( $charges ) ) {
							$email_message[] = sprintf( esc_html__( 'After deducting withdrawal charges, you\'ll receive amount %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $withdraw_amount_after_charges ) ) );
						}

						$email_message[] = sprintf( esc_html__( 'Withdrawal Note: %s', 'ddwc-wallet-management' ), $note );

						$sender_email = get_userdata( $user_id )->user_email;

						$email_data = [
							'email'   => $sender_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'You\'ve got new wallet withdrawal request of amount %s from %s. You can pay him/her manually on the below details then mark it as paid from withdraw requests menu.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ), '<a href="mailto:' . esc_attr( $sender_email ) . '">' . esc_html( $sender_email ) . '</a>' );

						if ( ! empty( $charges ) ) {
							$email_message[] = sprintf( esc_html__( 'After deducting withdrawal charges, you\'ll need to pay amount %s to the user.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $withdraw_amount_after_charges ) ) );
						}

						$email_message[] = sprintf( esc_html__( 'Withdrawal Note: %s', 'ddwc-wallet-management' ), $note );

						$email_data = [
							'email'   => get_option( 'admin_email' ),
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$response = [
							'success' => true,
							'message' => sprintf( esc_html__( 'Withdraw request for wallet amount %s has been made successfully.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $amount ) ) ),
						];
					}
				}
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
		 * Send OTP to user
		 *
		 * @return void
		 */
		public function ddwcwm_sent_otp_to_user() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				global $ddwcwm_wallet;

				$otp_length = $ddwcwm_wallet[ 'otp_length' ];
				$otp = rand( pow( 10, $otp_length - 1 ), pow( 10, $otp_length ) - 1 );

				$user_id = get_current_user_id();

				$email_message = [];

				$email_message[] = sprintf( esc_html__( 'Your verification OTP for wallet operation is %s', 'ddwc-wallet-management' ), $otp );

				$sender_email = get_userdata( $user_id )->user_email;

				$email_data = [
					'email'   => $sender_email,
					'message' => $email_message
				];

				do_action( 'ddwcwm_mail', $email_data );

				$message = esc_html__( 'OTP sent!!', 'ddwc-wallet-management' );

				if ( ! empty( $ddwcwm_wallet[ 'otp_expiry' ] ) ) {
					$message = sprintf( esc_html__( 'OTP sent and it will be expired in %s minute(s)', 'ddwc-wallet-management' ), round( $ddwcwm_wallet[ 'otp_expiry' ] / 60, 2 ) );
                }

				$response = [
					'success'           => true,
					'otp'               => base64_encode( $otp ),
					'otpExpiredMessage' => sprintf( esc_html__( 'OTP Expired!! %sResend OTP%s', 'ddwc-wallet-management' ), '<span class="button ddwcwm-resend-otp">', '</span>' ),
					'inValidOtpMessage' => esc_html__( 'OTP Invalid!!', 'ddwc-wallet-management' ),
					'message'           => $message,
				];
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
		 * Wallet refer to user
		 *
		 * @return void
		 */
		public function ddwcwm_wallet_refer_to_user() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				$email   = ! empty( $_POST[ 'ddwcwm_email' ] ) && is_email( $_POST[ 'ddwcwm_email' ] ) ? sanitize_text_field( $_POST[ 'ddwcwm_email' ] ) : '';
				$message = ! empty( $_POST[ 'ddwcwm_message' ] ) ? stripslashes_deep( sanitize_text_field( $_POST[ 'ddwcwm_message' ] ) ) : '';

				if ( empty( $email ) || empty( $message ) ) {
					$response = [
						'success' => false,
						'message' => esc_html__( 'Email or message fields are either empty or invalid.', 'ddwc-wallet-management' ),
					];
				} else {
					global $ddwcwm_wallet;

					$user_id  = get_current_user_id();
					$username = get_userdata( $user_id )->user_login;

					$email_message = [];

					$email_message[] = $message;

					$email_message[] = sprintf( esc_html__( 'Registration URL - %s.', 'ddwc-wallet-management' ), '<a href="' . esc_url( get_permalink( get_option( 'woocommerce_myaccount_page_id' ) ) . "?ddwcwm_referred=$username" ) . '">' . esc_url( get_permalink( get_option( 'woocommerce_myaccount_page_id' ) ) . "?ddwcwm_referred=$username" ) . '</a>' );

					$email_data = [
						'email'   => $email,
						'message' => $email_message
					];

					do_action( 'ddwcwm_mail', $email_data );

					$response = [
						'success' => true,
						'message' => esc_html__( 'Mail sent successfully.', 'ddwc-wallet-management' ),
					];
				}
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}
	}
}
