<?php
/**
 * Front hooks
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Front;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Front_Hooks' ) ) {
	/**
	 * Front hooks class
	 */
	class DDWCWM_Front_Hooks extends DDWCWM_Front_Functions {
		/**
		 * Construct
		 */
		public function __construct() {
			global $ddwcwm_wallet;

            if ( ! empty( get_option( '_ddwcwm_license_activated' ) ) && ! empty( $ddwcwm_wallet[ 'enabled' ] ) ) {
				add_filter( 'woocommerce_account_menu_items', [ $this, 'ddwcwm_add_woocommerce_menu' ] );

				add_filter( 'query_vars', [ $this, 'ddwcwm_add_query_vars' ] );

				add_action( 'woocommerce_account_' . $ddwcwm_wallet[ 'my_account_endpoint' ] . '_endpoint', [ $this, 'ddwcwm_add_wallet_content_on_my_account_page' ] );

				add_filter( 'sidebars_widgets', [ $this, 'ddwcwm_remove_sidebar_from_wallet_page' ] );

				add_action( 'wp_enqueue_scripts', [ $this, 'ddwcwm_register_front_scripts' ] );

				add_action( 'template_redirect', [ $this, 'ddwcwm_display_notice_if_wallet_topup_added_in_cart' ] );

				add_filter( 'woocommerce_add_to_cart_validation', [ $this, 'ddwcwm_check_if_wallet_topup_product_exists_in_cart' ] );

				add_filter( 'woocommerce_before_cart', [ $this, 'ddwcwm_check_if_wallet_topup_product_exists_in_cart' ] );

				add_filter( 'woocommerce_available_payment_gateways', [ $this, 'ddwcwm_remove_payment_gateway' ] );

				if ( is_user_logged_in() ) {
					add_action( 'woocommerce_review_order_after_order_total', [ $this, 'ddwcwm_add_wallet_template_on_checkout_page' ] );

					add_action( 'woocommerce_cart_calculate_fees', [ $this, 'ddwcwm_add_wallet_as_fee_in_cart' ] );

					add_filter( 'woocommerce_cart_totals_fee_html', [ $this, 'ddwcwm_add_remove_wallet_button_on_checkout' ], 10, 2 );
				}

				// add_action( 'woocommerce_checkout_order_processed', [ $this, 'ddwcwm_deduct_wallet_amount_on_order_processed' ] );

				add_action( 'woocommerce_order_status_on-hold', [ $this, 'ddwcwm_deduct_wallet_amount_on_order_processed' ] );
				add_action( 'woocommerce_order_status_processing', [ $this, 'ddwcwm_deduct_wallet_amount_on_order_processed' ] );

				add_filter( 'woocommerce_cart_totals_get_fees_from_cart_taxes', [ $this, 'ddwcwm_remove_tax_from_wallet_fee' ], 10, 2 );

                add_action( 'woocommerce_before_calculate_totals', [ $this, 'ddwcwm_woocommerce_before_calculate_totals' ] );

				add_action( 'wp_loaded', [ $this, 'ddwcwm_handle_wallet_topup' ] );

				// Shortcodes
				add_shortcode( $this->ddwcwm_validate_shortcode( $ddwcwm_wallet[ 'wallet_balance_only_shortcode' ] ), [ $this, 'ddwcwm_add_wallet_balance_shortcode_content' ] );

				add_shortcode( $this->ddwcwm_validate_shortcode( $ddwcwm_wallet[ 'wallet_balance_shortcode' ] ), [ $this, 'ddwcwm_add_wallet_balance_layout_shortcode_content' ] );

				add_shortcode( $this->ddwcwm_validate_shortcode( $ddwcwm_wallet[ 'wallet_operations_shortcode' ] ), [ $this, 'ddwcwm_add_wallet_operations_shortcode_content' ] );

				add_shortcode( $this->ddwcwm_validate_shortcode( $ddwcwm_wallet[ 'wallet_balance_operations_shortcode' ] ), [ $this, 'ddwcwm_add_wallet_balance_operations_shortcode_content' ] );

				add_shortcode( $this->ddwcwm_validate_shortcode( $ddwcwm_wallet[ 'wallet_transactions_shortcode' ] ), [ $this, 'ddwcwm_add_wallet_transactions_shortcode_content' ] );
			}
		}

		/**
		 * Validate shortcode function
		 *
		 * @param string $shortcode
		 * @return string
		 */
		public function ddwcwm_validate_shortcode( $shortcode ) {
			$shortcode = str_replace( '[', '', $shortcode );
			$shortcode = str_replace( ']', '', $shortcode );
			$shortcode = str_replace( ' ', '', $shortcode );

			return $shortcode;
		}
	}
}
